import 'dart:math';
import 'dart:ui';

/// Package imports
import 'package:flutter/material.dart';
import 'package:intl/intl.dart' show DateFormat;
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_charts/charts.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatelessWidget {
  const MyApp({Key? key}) : super(key: key);

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    return const MaterialApp(
      debugShowCheckedModeBanner: false,
      home: LineCustomization(),
    );
  }
}

class LineCustomization extends StatefulWidget {
  const LineCustomization({super.key});

  /// Creates the customized Line chart.

  @override
  LineCustomizationState createState() => LineCustomizationState();
}

/// State class of customized Line chart.
class LineCustomizationState extends State<LineCustomization> {
  LineCustomizationState();
  List<DataPoints> data = [
    DataPoints(DateTime(2022, 10, 1), 30),
    DataPoints(DateTime(2022, 10, 2), 40),
    DataPoints(DateTime(2022, 10, 3), 90),
    DataPoints(DateTime(2022, 10, 4), 60),
    DataPoints(DateTime(2022, 10, 6), 20),
    DataPoints(DateTime(2022, 10, 7), 50),
    DataPoints(DateTime(2022, 10, 8), 45),
    DataPoints(DateTime(2022, 10, 9), 50),
    DataPoints(DateTime(2022, 10, 10), 35),
    DataPoints(DateTime(2022, 10, 10), null),
    DataPoints(DateTime(2022, 10, 14), 20),
    DataPoints(DateTime(2022, 10, 15), 90),
  ];

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SfCartesianChart(
        plotAreaBorderWidth: 0,
        primaryXAxis: DateTimeAxis(
          axisLine: const AxisLine(
            color: Colors.blueGrey,
            width: 2,
          ),
          majorTickLines: const MajorTickLines(width: 0),
          interval: 1,
          plotOffset: 30,
          majorGridLines: const MajorGridLines(
            width: 1,
            color: Colors.blueGrey,
            dashArray: <double>[3, 3],
          ),
          dateFormat: DateFormat.d(),
        ),
        primaryYAxis: NumericAxis(
          axisLine: const AxisLine(
            color: Colors.blueGrey,
            width: 2,
          ),
          majorGridLines: const MajorGridLines(width: 0),
          majorTickLines: const MajorTickLines(width: 0),
          axisLabelFormatter: (args) {
            return ChartAxisLabel(
                args.text == args.text ? "" : args.text, args.textStyle);
          },
        ),
        series: <ChartSeries<DataPoints, DateTime>>[
          LineSeries<DataPoints, DateTime>(
            animationDuration: 0,
            dataSource: data,
            xValueMapper: (DataPoints sales, _) => sales.x as DateTime,
            yValueMapper: (DataPoints sales, _) => sales.y,
            width: 2,
            dashArray: const <double>[10, 5],
            markerSettings: const MarkerSettings(
              width: 5,
              height: 5,
              isVisible: true,
              shape: DataMarkerType.circle,
              color: Colors.blueGrey,
              borderColor: Colors.blueGrey,
            ),
            onCreateRenderer: (ChartSeries<dynamic, dynamic> series) {
              return _CustomLineSeriesRenderer(
                  series as LineSeries<DataPoints, DateTime>);
            },
          ),
        ],
        onMarkerRender: (MarkerRenderArgs args) {
          if (args.pointIndex == 1 ||
              args.pointIndex == 4 ||
              args.pointIndex == 5 ||
              args.pointIndex == 11) {
            args.markerHeight = 13;
            args.markerWidth = 13;
          } else if (args.pointIndex == 8 ||
              args.pointIndex == 3 ||
              args.pointIndex == 10) {
            args.markerHeight = 20;
            args.markerWidth = 20;
          }
        },
      ),
    );
  }
}

class DataPoints {
  DataPoints(this.x, this.y);

  final DateTime? x;
  final double? y;
}

/// custom Line series class overriding the original Line series class.
class _CustomLineSeriesRenderer extends LineSeriesRenderer {
  _CustomLineSeriesRenderer(this.series);

  final LineSeries<dynamic, dynamic> series;

  @override
  LineSegment createSegment() {
    return _LineCustomPainter(series);
  }
}

late List<double> yValue;
late List<double> xValue;

/// custom Line painter class for customized Line series.
class _LineCustomPainter extends LineSegment {
  _LineCustomPainter(this.series) {
    yValue = <double>[];
    xValue = <double>[];
  }

  final LineSeries<dynamic, dynamic> series;

  @override
  int get currentSegmentIndex => super.currentSegmentIndex!;

  @override
  Paint getStrokePaint() {
    final Paint customerStrokePaint = Paint();
    customerStrokePaint.color = Colors.blueGrey;
    customerStrokePaint.strokeWidth = 2;
    customerStrokePaint.style = PaintingStyle.stroke;
    return customerStrokePaint;
  }

  @override
  void onPaint(Canvas canvas) {
    final double x1 = points[0].dx,
        y1 = points[0].dy,
        x2 = points[1].dx,
        y2 = points[1].dy;
    yValue.add(y1);
    yValue.add(y2);
    xValue.add(x1);
    xValue.add(x2);
    final Path path = Path();
    path.moveTo(x1, y1);
    path.lineTo(x2, y2);
    if (currentSegmentIndex == 3) {
      _drawDashedLine(canvas, series, strokePaint!, path, true);
    } else {
      canvas.drawPath(path, getStrokePaint());
    }
  }
}

void _drawDashedLine(Canvas canvas, CartesianSeries<dynamic, dynamic> series,
    Paint paint, Path path, bool isSeries) {
  bool _even = false;
  for (int i = 1; i < series.dashArray.length; i = i + 2) {
    if (series.dashArray[i] == 0) {
      _even = true;
    }
  }
  if (_even == false) {
    paint.isAntiAlias = true;
    canvas.drawPath(
        _dashPath(
          path,
          dashArray: _CircularIntervalList<double>(
              isSeries ? series.dashArray : <double>[12, 3, 3, 3]),
        )!,
        paint);
  } else {
    canvas.drawPath(path, paint);
  }
}

Path? _dashPath(
  Path source, {
  required _CircularIntervalList<double> dashArray,
}) {
  if (source == null) {
    return null;
  }
  const double _intialValue = 0.0;
  final Path _path = Path();
  for (final PathMetric measurePath in source.computeMetrics()) {
    double _distance = _intialValue;
    bool _draw = true;
    while (_distance < measurePath.length) {
      final double length = dashArray.next;
      if (_draw) {
        _path.addPath(measurePath.extractPath(_distance, _distance + length),
            Offset.zero);
      }
      _distance += length;
      _draw = !_draw;
    }
  }
  return _path;
}

class _CircularIntervalList<T> {
  _CircularIntervalList(this._values);
  final List<T> _values;
  int _index = 0;
  T get next {
    if (_index >= _values.length) {
      _index = 0;
    }
    return _values[_index++];
  }
}
